@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import sys
 from contextlib import suppress
 from functools import wraps
 from inspect import Signature
@@ -22,7 +23,7 @@ from typing import (
     overload,
 )
 
-from typing_extensions import Self, deprecated
+from typing_extensions import Self, TypeVar, deprecated
 
 from .builtins import F0, F1, P0, P1, R0, R1, T0, KwargsT, P, R, R0_co, R1_co, R_co, T, T0_co, T1_co, T_co
 
@@ -42,9 +43,9 @@ __all__ = [
 # ruff: noqa: N801
 
 
-class copy_signature(Generic[F0]):
+def copy_signature(target: F0, /) -> Callable[[Callable[..., Any]], F0]:
     """
-    Type util to copy the signature of one function to another function.
+    Utility function to copy the signature of one function to another one.
 
     Especially useful for passthrough functions.
 
@@ -70,12 +71,11 @@ class copy_signature(Generic[F0]):
                # another thing
     """
 
-    def __init__(self, target: F0) -> None:
-        """Copy the signature of ``target``."""
-
-    def __call__(self, wrapped: Callable[..., Any]) -> F0:
+    def decorator(wrapped: Callable[..., Any]) -> F0:
         return cast(F0, wrapped)
 
+    return decorator
+
 
 class injected_self_func(Protocol[T_co, P, R_co]):
     @overload
@@ -516,7 +516,10 @@ class classproperty(classproperty_base[T, R_co]):
                     del cache[name]
 
 
-class cachedproperty(property, Generic[R_co]):
+_T_cc = TypeVar("_T_cc", default=Any)
+
+
+class cachedproperty(property, Generic[R_co, _T_cc]):
     """
     Wrapper for a one-time get property, that will be cached.
 
@@ -538,16 +541,22 @@ class cachedproperty(property, Generic[R_co]):
         def __init__(
             self,
             fget: Callable[[Any], R_co],
-            fset: Callable[[Any, Any], None] | None = None,
+            fset: Callable[[Any, _T_cc], None] | None = None,
             fdel: Callable[[Any], None] | None = None,
             doc: str | None = None,
         ) -> None: ...
 
-        def getter(self, fget: Callable[..., R_co]) -> cachedproperty[R_co]: ...
+        def getter(self, fget: Callable[..., R_co]) -> cachedproperty[R_co, _T_cc]: ...
 
-        def setter(self, fset: Callable[[Any, Any], None]) -> cachedproperty[R_co]: ...
+        def setter(self, fset: Callable[[Any, _T_cc], None]) -> cachedproperty[R_co, _T_cc]: ...
 
-        def deleter(self, fdel: Callable[..., None]) -> cachedproperty[R_co]: ...
+        def deleter(self, fdel: Callable[..., None]) -> cachedproperty[R_co, _T_cc]: ...
+
+    if sys.version_info < (3, 13):
+
+        def __init__(self, fget: Any, fset: Any | None = None, fdel: Any | None = None, doc: str | None = None) -> None:
+            self.__name__ = fget.__name__
+            super().__init__(fget, fset, fdel, doc)
 
     @overload
     def __get__(self, instance: None, owner: type | None = None) -> Self: ...
@@ -566,7 +575,7 @@ class cachedproperty(property, Generic[R_co]):
         cache[self.__name__] = value
         return value
 
-    def __set__(self, instance: Any, value: Any) -> None:
+    def __set__(self, instance: Any, value: _T_cc) -> None:
         if self.__name__ in (cache := instance.__dict__.setdefault(self.cache_key, {})):
             del cache[self.__name__]
 
@@ -598,6 +607,17 @@ class cachedproperty(property, Generic[R_co]):
             with suppress(KeyError):
                 del cache[name]
 
+    @classmethod
+    def update_cache(cls, obj: object, name: str, value: Any) -> None:
+        """
+        Update cached property of an object instance.
+
+        :param obj:   The object whose cache should be updated.
+        :param names: Property name to update.
+        :param value: The value to assign.
+        """
+        obj.__dict__.setdefault(cls.cache_key, {})[name] = value
+
 
 class KwargsNotNone(KwargsT):
     """Remove all None objects from this kwargs dict."""
